@@ -2,28 +2,33 @@ mod log;
 mod utils;
 
 use clap::Parser;
+use dirs::home_dir;
 use git2::{Branch, Repository};
 use log::{println, println_label, OutputLabel};
 use std::{path::Path, time::Instant};
 use utils::{
+	config::get_related_config,
 	crawl::crawl_directory_for_repos,
 	git::{find_ahead_branches_in_repo, is_repo_dirty},
 };
 use yansi::Paint;
 
-// TODO: proper unwrap handling
-
 /// Push all commits in git repositories
-#[derive(Parser, Debug)]
-#[clap(name = "git-leave", about, version, author)]
+#[derive(Parser)]
+#[clap(name = "git-leave", about, version, author, long_about = None)]
 struct Arguments {
 	/// The directory to search in
 	#[clap(default_value_t = String::from("."))]
 	directory: String,
 
-	/// Push commits to remote
+	// TODO
+	/// Don't trim output path (may result in weird behavior on screen)
 	#[clap(short, long)]
-	push: bool,
+	no_trim: bool,
+
+	/// Use git config default folder value for the directory to search in
+	#[clap(short, long)]
+	default: bool,
 }
 
 fn main() {
@@ -33,26 +38,68 @@ fn main() {
 		Paint::disable();
 	}
 
-	// Parse command line arguments
+	// Parse command line arguments and get related config
 	let args = Arguments::parse();
+	let config = get_related_config();
 
 	// Display the name of the program
 	println_label(
 		OutputLabel::Success("Welcome"),
-		format!("to {}", Paint::yellow("git-leave")),
+		format!("to {}", Paint::yellow("git leave")),
 	);
 
+	// Set the path to the one specified in the global config
+	// only if the default argument is enabled,
+	// else set to the path specified in the arguments.
+	let mut path = match config {
+		Some(conf) => match (args.default, conf.default_folder) {
+			(true, Some(dir)) => dir,
+			(true, None) => {
+				println_label(OutputLabel::Warning, "No default folder set in config");
+
+				args.directory
+			}
+			(_, _) => args.directory,
+		},
+		_ => args.directory,
+	};
+
+	path = path.replace("~", home_dir().unwrap().to_str().unwrap());
+
 	// Get absolute path
-	let search_directory = Path::new(&args.directory)
-		.canonicalize()
-		.expect("Could not get absolute path");
+	let search_directory = match Path::new(&path).canonicalize() {
+		Ok(path) => path,
+		Err(err) => {
+			println_label(
+				OutputLabel::Error,
+				format!(
+					"Could not get absolute path of specified directory: {}",
+					err
+				),
+			);
+
+			return;
+		}
+	};
 
 	// Start the timer
 	let begin_search_time = Instant::now();
 
 	// Find git repositories in the specified directory
-	let repos =
-		crawl_directory_for_repos(&search_directory).expect("Could not read folder content");
+	let repos = match crawl_directory_for_repos(&search_directory) {
+		Ok(repos) => repos,
+		Err(err) => {
+			println_label(
+				OutputLabel::Error,
+				format!(
+					"Something went wrong while trying to crawl the directory: {}",
+					err
+				),
+			);
+
+			return;
+		}
+	};
 
 	// Exit if no git repositories were found
 	if repos.is_empty() {
@@ -86,7 +133,7 @@ fn main() {
 					.unwrap()
 					.to_str()
 					.unwrap()
-					.replace(env!("HOME"), "~"),
+					.replace(home_dir().unwrap().as_path().to_str().unwrap(), "~"),
 			);
 		});
 	}
@@ -123,7 +170,7 @@ fn main() {
 					Paint::yellow(
 						ahead_branches
 							.iter()
-							.map(|branch| branch.name().unwrap().unwrap())
+							.map(|branch| branch.name().unwrap().unwrap_or("<no name found>"))
 							.collect::<Vec<&str>>()
 							.join("/")
 					)
